#!/bin/sh
if [ "$USER" != "root" ] ; then
  echo "Somente o root pode executar essa operação!";
  echo "Pressione [ENTER] para retornar."
  read espera
  exit 0;
fi

#
# determinando localizacao e nome do script
#
SCRIPT_HDD_SPIN_FIX="/tmp/99-hdd-spin-fix.sh"
if [ -f "$SCRIPT_HDD_SPIN_FIX" ] ; then
  rm -f /tmp/99-hdd-spin-fix.sh
fi

#
# variaveis necessarias
#
NEED_HOTFIX="N"
TIME_TEST="30"      ## tempo para teste em minutos
TIME_TEST_UNIT="m"  ## s=segundos, m=minutos

#
# hdparm esta instalado ?
#
echo -n "procurando o aplicativo [hdparm]..."
hdparm -V
if [ $? -ne 0 ] ; then
    echo "O hdparm não está instalado ou não foi encontrado no path do sistema."
    echo "Experimente instala-lo :"
    echo "sudo apt-get install hdparm"
    echo "Pressione [ENTER] para retornar."
    read espera
    exit 2;
fi

#
# smarttools esta instalado ?
#
echo -n "procurando o aplicativo [smartctl]..."
smartctl --version|grep "release"
if [ $? -ne 0 ] ; then
    echo "O smartctl não está instalado ou não foi encontrado no path do sistema."
    echo "Experimente instala-lo :"
    echo "sudo apt-get install smartmontools"
    echo "Pressione [ENTER] para retornar."
    read espera
    exit 2;
fi

#
# laptop-mode-tools  esta instalado ?
#
echo -n "procurando o aplicativo [laptop-mode-tools]..."
if ! [ -f "/etc/laptop-mode/laptop-mode.conf" ] ; then
    echo "O laptop-mode-tools não está instalado ou não foi encontrado no path do sistema."
    echo "Experimente instala-lo :"
    echo "sudo apt-get install laptop-mode-tools"
    echo "Pressione [ENTER] para retornar."
    read espera
    exit 2;
fi

#
# vai haver bastante verbose entao é recomendado limpar a tela
#
clear

#
# Buscando saber quais unidades possuo
#
HDD_LIST="`ls -1 /dev/sd*|grep -v \"1\"|grep -v \"2\"|grep -v \"3\"|grep -v \"4\"`"

#
# Testando cada unidade por cerca de $TIME_TEST minutos
#
for DISK in $HDD_LIST ; do 
  #
  # Exibindo a unidade que estará sendo monitorada
  #
  echo "====================== $DISK ========================="

  #
  # monitorando a unidade por $TIME_TEST minutos
  # 
  CICLES_NUMBER_START="`smartctl -A $DISK | grep Load_Cycle_Count|cut -d \"-\" -f 2`"
  echo "O numero de ciclos de carga/descarga atual para a unidade :"
  echo "$DISK é : $CICLES_NUMBER_START"
  echo "Vamos aguardar $TIME_TEST minutos para ver o numero de ciclos"
  echo "que será feita com essa unidade. Mantenha este computador desocupado,"
  echo " isto é, sem nenhum tipo de uso que possa prejudicar o teste."
  #
  # dormindo por $TIME_TEST minutos
  # 
  sleep "$TIME_TEST$TIME_TEST_UNIT"

  #
  # calculando quantidade de ciclos de liga/desliga
  #  
  echo "Hummmm, hora de acordar. Vamos ver como está o numero de ciclos agora..."
  CICLES_NUMBER_FINISH="`smartctl -A $DISK | grep Load_Cycle_Count|cut -d \"-\" -f 2`"

  # debug - apenas para a fase de testes
  #CICLES_NUMBER_START="   125043"   ## debug
  #CICLES_NUMBER_FINISH="   126043"  ## debug

  CICLES_DIFF=$((CICLES_NUMBER_FINISH - CICLES_NUMBER_START))
  CICLES_DIFF_BY_MIN=$(($CICLES_DIFF / $TIME_TEST))
  echo "O numero de ciclos de carga/descarga atualizado para a unidade :"
  echo "$DISK é : $CICLES_NUMBER_FINISH"
  echo "O numero de ciclos de carga/descarga feito no período de"
  echo "$TIME_TEST minutos foi de $CICLES_DIFF ciclos"
  echo "A unidade $DISK esta fazendo $CICLES_DIFF_BY_MIN ciclo(s) de carga/descarga por minuto"
  #
  # Se detectar que precisa de reparos, então
  # fazemos os ajustes necessarios
  #
  if [ "$CICLES_DIFF_BY_MIN" -gt 0 ]; then
     # PARAM=254 mantem um gerenciamento de energia menos agressivo
     # PARAM=255 desliga completamente o gerenciamento de energia do HD
     # Qualquer um deles poderá ser usado no caso do sistema sugerir
     # a manutenção dos discos. Porem a resposta padrao sera :
     PARAM=255
     echo "Estou recomendando reparar o ciclo de carga/descarga para a unidade $DISK"
     echo "Há dois tipos diferentes de reparação :"
     echo "1- Desligar completamentamente o gerenciamento de energia do HD"
     echo "   Assim, os ciclos de carga serão mínimos,"
     echo "   mas o HD ficará a uma temperatura mais elevada que o normal."
     echo "2- Manter o gerenciamento de energia do HD, porém menos agressivo."
     echo "   Assim, os ciclos de carga serão menores, mas ainda continuará"
     echo "   com ciclos que carga/desgarga."
     echo "Recomendação : use a opção [1], pois é consenso que o HD pode lidar melhor"
     echo "com alguns graus de temperatura a mais do que muitos ciclos de carga."
     echo "---" 
     echo -n "Escolha qual a reparação desejada (1 ou 2):"
     read RESPOSTA
     if [ "$RESPOSTA" = "2" ] ; then
        PARAM=254
        echo "Voce escolheu :"
        echo "2- Manter o gerenciamento de energia do HD, porém menos agressivo."
     fi
     echo "Executando o procedimento de reparação escolhido..."

     if ! [ -f "$SCRIPT_HDD_SPIN_FIX" ] ; then
       if [ "$NEED_HOTFIX" = "N" ] ; then
          NEED_HOTFIX="S"
          echo "#!/bin/sh" >$SCRIPT_HDD_SPIN_FIX
       fi
     fi
     echo "hdparm -B $PARAM $DISK" >>$SCRIPT_HDD_SPIN_FIX
  fi
done

if [ "$NEED_HOTFIX" != "S" ] ; then
  echo "Seu sistema não precisa ser reparado, seus discos estao funcionando OK."
  echo "Pressione [ENTER] para retornar."
  read espera
  exit 0;
fi

#
# habilitando um script que no momento de boot
# irá desabilitar o gerenciamento de energia dos discos
#
chmod a+x $SCRIPT_HDD_SPIN_FIX

#
# copiando o script para o lugar em que deveriam ficar
#
[ -d "/etc/acpi/suspend.d/" ] && cp $SCRIPT_HDD_SPIN_FIX /etc/acpi/suspend.d/
[ -d "/etc/acpi/resume.d/" ] && cp $SCRIPT_HDD_SPIN_FIX /etc/acpi/resume.d/
[ -d "/etc/acpi/start.d/" ] && mv $SCRIPT_HDD_SPIN_FIX /etc/acpi/start.d/

if [ -f /etc/laptop-mode/laptop-mode.conf ] ; then
  echo "# Ubuntu Perfeito adicionou essas linhas"
  echo "LM_AC_HD_POWERMGMT=$PARAM" >>/etc/laptop-mode/laptop-mode.conf 
  echo "NOLM_AC_HD_POWERMGMT=$PARAM" >>/etc/laptop-mode/laptop-mode.conf 
fi

echo "Seu sistema foi ajustado e fará afeito após reiniciar o computador !"

# Fim do script
